import 'dart:convert';

import 'package:flutter/material.dart';
import 'constants.dart';
import 'control.dart';

class CheckButton extends StatefulWidget {
  final String name;
  final dynamic value;
  final ValueChanged<dynamic> onChanged;
  final bool isRequired;
  final bool multiple;
  final Color color;
  final Color selectedColor;
  final String dataValue;
  final String dataLabel;
  final List<Map<String, dynamic>> options;
  final int height;
  final YekongaForm formControl;
  final WrapAlignment alignment;
  final double margin;

  CheckButton({
    Key key,
    this.formControl,
    @required this.name,
    @required this.options,
    this.value,
    this.isRequired: false,
    this.onChanged,
    this.multiple: false,
    this.color,
    this.alignment: WrapAlignment.spaceEvenly,
    this.selectedColor: Const.COLOR_PRIMARY,
    this.dataValue: 'value',
    this.dataLabel: 'label',
    this.height: 100,
    this.margin: 0,
  }) : super(key: key);

  @override
  _CheckButtonState createState() => _CheckButtonState();
}

class _CheckButtonState extends State<CheckButton> {
  dynamic innerValue;
  FocusNode focusNode;
  bool isValid = true;
  InputDecoration decoration;
  TextEditingController editingController = new TextEditingController();

  @override
  void initState() {
    super.initState();
    dynamic value;
    if (widget.value != null) {
      if (widget.multiple) {
        if (widget.value is List<dynamic>) {
          value = widget.value;
        } else if (widget.value is String) {
          try {
            value = List<dynamic>.from(jsonDecode(widget.value));
          } catch (e) {}
        }
      } else {
        value = widget.value;
      }
    }

    if (widget.formControl != null) {
      if (widget.formControl.containsKey(widget.name)) {
        value = widget.formControl.getValue(widget.name);
      } else {
        widget.formControl.setValue(widget.name, value);
      }

      widget.formControl.setClearFunction(widget.name, () {
        innerValue = (widget.multiple) ? [] : null;
        editingController.text = "";
        setState(() {});
      });

      focusNode = null;
      widget.formControl
          .setValidationRequirement(widget.name, widget.isRequired);
      widget.formControl.setFocusNode(widget.name, focusNode);
    }

    innerValue = value;
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      margin: EdgeInsets.only(bottom: widget.margin),
      child: Wrap(
        spacing: 5.0,
        alignment: widget.alignment,
        children: List.generate(
          widget.options.length,
          (i) {
            Map<String, dynamic> item = widget.options[i];

            return checkButtonObtion(item);
          },
        ),
      ),
    );
  }

  checkButtonObtion(Map<String, dynamic> item) {
    bool itemSelected = (widget.multiple)
        ? innerValue.contains(item[widget.dataValue])
        : (innerValue == item[widget.dataValue]);

    return ChoiceChip(
      padding: EdgeInsets.all(10),
      // selectedColor: widget.selectedColor,
      backgroundColor: (itemSelected) ? widget.selectedColor : widget.color,
      label: Text(
        item[widget.dataLabel],
        style: TextStyle(
          color: Const.COLOR_TEXT,
        ),
      ),
      selected: itemSelected,
      onSelected: (final bool _selected) {
        setState(
          () {
            if (widget.multiple) {
              if (_selected && !innerValue.contains(item[widget.dataValue])) {
                if (!widget.multiple) {
                  innerValue.clear();
                }
                innerValue.add(item[widget.dataValue]);
              } else if (!_selected &&
                  innerValue.contains(item[widget.dataValue])) {
                innerValue.remove(item[widget.dataValue]);
              }
            } else {
              if (_selected) {
                innerValue = item[widget.dataValue];
              }
            }
            print("Yekonga:  " + innerValue.toString());
            if (widget.formControl != null) {
              widget.formControl.setValue(widget.name, innerValue);
            }

            if (widget.onChanged != null) widget.onChanged(innerValue);
          },
        );
      },
    );
  }
}
