import 'package:flutter/material.dart';
import './control.dart';

class FormCheckbox extends StatefulWidget {
  final String name;
  final bool value;
  final ValueChanged<bool> onChanged;
  final Color activeColor;
  final Color checkColor;
  final Color tileColor;
  final Widget title;
  final Widget subtitle;
  final Widget secondary;
  final bool isThreeLine;
  final bool isRequired;
  final bool dense;
  final bool selected;
  final ListTileControlAffinity controlAffinity;
  final bool autofocus;
  final EdgeInsetsGeometry contentPadding;
  final bool tristate;
  final ShapeBorder shape;
  final Color selectedTileColor;
  final YekongaForm formControl;

  FormCheckbox({
    Key key,
    this.formControl,
    @required this.name,
    @required this.value,
    @required this.onChanged,
    this.activeColor,
    this.checkColor,
    this.tileColor,
    this.title,
    this.subtitle,
    this.isRequired = false,
    this.isThreeLine = false,
    this.dense,
    this.secondary,
    this.selected = false,
    this.controlAffinity = ListTileControlAffinity.platform,
    this.autofocus = false,
    this.contentPadding = EdgeInsets.zero,
    this.tristate = false,
    this.shape,
    this.selectedTileColor,
  }) : super(key: key);

  @override
  _FormCheckboxState createState() => _FormCheckboxState();
}

class _FormCheckboxState extends State<FormCheckbox> {
  bool innerValue = false;
  FocusNode focusNode;
  bool isValid = true;
  InputDecoration decoration;
  TextEditingController editingController = new TextEditingController();

  @override
  void initState() {
    super.initState();
    if (widget.formControl != null) {
      if (widget.formControl.containsKey(widget.name)) {
        innerValue = widget.formControl.getValue(widget.name);
      }

      widget.formControl.setClearFunction(widget.name, () {
        innerValue = false;
        editingController.text = "";
        setState(() {});
      });

      focusNode = null;
      widget.formControl
          .setValidationRequirement(widget.name, widget.isRequired);
      widget.formControl.setFocusNode(widget.name, focusNode);
    }

    innerValue = widget.value;
  }

  @override
  Widget build(BuildContext context) {
    return CheckboxListTile(
      value: innerValue,
      onChanged: (value) {
        setState(() {
          innerValue = value;
          widget.formControl.setValue(widget.name, innerValue);
          widget.onChanged(innerValue);
        });
      },
      activeColor: widget.activeColor,
      checkColor: widget.checkColor,
      tileColor: widget.tileColor,
      title: widget.title,
      subtitle: widget.subtitle,
      isThreeLine: widget.isThreeLine,
      dense: widget.dense,
      secondary: widget.secondary,
      selected: widget.selected,
      controlAffinity: widget.controlAffinity,
      autofocus: widget.autofocus,
      contentPadding: widget.contentPadding,
      tristate: widget.tristate,
      shape: widget.shape,
      selectedTileColor: widget.selectedTileColor,
    );
  }
}
