import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';
import 'package:flutter/services.dart';
import 'constants.dart';
import './control.dart';

class FormDatePicker extends StatefulWidget {
  final YekongaForm formControl;
  final String name;
  final String value;
  final FocusNode focusNode;
  final InputDecoration decoration;
  final TextInputType keyboardType;
  final TextInputAction textInputAction;
  final TextCapitalization textCapitalization;
  final bool isRequired;
  final TextStyle style;
  final StrutStyle strutStyle;
  final TextAlign textAlign;
  final TextAlignVertical textAlignVertical;
  final TextDirection textDirection;
  final bool autofocus;
  final String obscuringCharacter;
  final bool obscureText;
  final bool autocorrect;
  final SmartDashesType smartDashesType;
  final SmartQuotesType smartQuotesType;
  final bool enableSuggestions;
  final int maxLines;
  final int minLines;
  final bool expands;
  final bool readOnly;
  final ToolbarOptions toolbarOptions;
  final bool showCursor;
  static const int noMaxLength = -1;
  final int maxLength;
  final bool maxLengthEnforced;
  final MaxLengthEnforcement maxLengthEnforcement;
  final ValueChanged<String> onChanged;
  final VoidCallback onEditingComplete;
  final ValueChanged<String> onSubmitted;
  final AppPrivateCommandCallback onAppPrivateCommand;
  final List<TextInputFormatter> inputFormatters;
  final bool enabled;
  final double cursorWidth;
  final double cursorHeight;
  final Radius cursorRadius;
  final Color cursorColor;
  final Brightness keyboardAppearance;
  final EdgeInsets scrollPadding;
  final bool enableInteractiveSelection;
  final TextSelectionControls selectionControls;
  final DragStartBehavior dragStartBehavior;
  bool get selectionEnabled => enableInteractiveSelection;
  final GestureTapCallback onTap;
  final MouseCursor mouseCursor;
  final InputCounterWidgetBuilder buildCounter;
  final ScrollPhysics scrollPhysics;
  final ScrollController scrollController;
  final Iterable<String> autofillHints;
  final String restorationId;

  FormDatePicker({
    Key key,
    this.formControl,
    this.value,
    @required this.name,
    this.focusNode,
    this.decoration = const InputDecoration(),
    this.keyboardType = TextInputType.text,
    this.textInputAction,
    this.textCapitalization = TextCapitalization.none,
    this.style,
    this.strutStyle,
    this.textAlign = TextAlign.start,
    this.textAlignVertical,
    this.textDirection,
    this.readOnly = true,
    this.toolbarOptions,
    this.showCursor,
    this.isRequired = false,
    this.autofocus = false,
    this.obscuringCharacter = '•',
    this.obscureText = false,
    this.autocorrect = true,
    this.smartDashesType = SmartDashesType.enabled,
    this.smartQuotesType = SmartQuotesType.enabled,
    this.enableSuggestions = true,
    this.maxLines = 1,
    this.minLines,
    this.expands = false,
    this.maxLength,
    this.maxLengthEnforced = true,
    this.maxLengthEnforcement,
    this.onChanged,
    this.onEditingComplete,
    this.onSubmitted,
    this.onAppPrivateCommand,
    this.inputFormatters,
    this.enabled = true,
    this.cursorWidth = 2.0,
    this.cursorHeight,
    this.cursorRadius,
    this.cursorColor,
    this.keyboardAppearance,
    this.scrollPadding = const EdgeInsets.all(20.0),
    this.dragStartBehavior = DragStartBehavior.start,
    this.enableInteractiveSelection = true,
    this.selectionControls,
    this.onTap,
    this.mouseCursor,
    this.buildCounter,
    this.scrollController,
    this.scrollPhysics,
    this.autofillHints,
    this.restorationId,
  }) : super(key: key);

  @override
  _FormDatePickerState createState() => _FormDatePickerState();
}

class _FormDatePickerState extends State<FormDatePicker> {
  DateTime innerValue;
  FocusNode focusNode;
  bool isValid = true;
  InputDecoration decoration;
  TextEditingController editingController = new TextEditingController();

  Future<void> _selectDate(BuildContext context) async {
    DateTime _value = (innerValue != null) ? innerValue : DateTime.now();
    final DateTime value = await showDatePicker(
      context: context,
      initialDate: _value,
      firstDate: DateTime(1980, 1),
      lastDate: DateTime(2101),
    );

    if (value != null && value != innerValue) {
      setState(() {
        innerValue = value;
        String strValue = getValue(innerValue);
        widget.formControl.setValue(widget.name, strValue);
        if (widget.onChanged != null) widget.onChanged(strValue);
        editingController.text = strValue;
      });
    }
  }

  @override
  void initState() {
    super.initState();
    dynamic value = getValue(innerValue);

    if (widget.formControl != null) {
      if (widget.formControl.containsKey(widget.name)) {
        value = widget.formControl.getValue(widget.name);
      } else {
        widget.formControl.setValue(widget.name, value);
      }

      if (value is String) {
        innerValue = DateTime.tryParse(value);
      }

      widget.formControl.setClearFunction(widget.name, () {
        innerValue = null;
        editingController.text = "";
        setState(() {});
      });

      focusNode = widget.focusNode ?? FocusNode();
      widget.formControl
          .setValidationRequirement(widget.name, widget.isRequired);
      widget.formControl.setFocusNode(widget.name, focusNode);
    }

    editingController.text = value.toString();

    decoration = widget.decoration.copyWith(
      suffixIcon:
          (widget.decoration != null && widget.decoration.prefixIcon != null)
              ? null
              : Padding(
                  padding: const EdgeInsetsDirectional.only(end: 12.0),
                  child: Icon(
                    Icons.calendar_today_outlined,
                  ), // myIcon is a 48px-wide widget.
                ),
      prefixIcon:
          (widget.decoration != null) ? widget.decoration.prefixIcon : null,
      floatingLabelBehavior: (widget.decoration != null)
          ? widget.decoration.floatingLabelBehavior
          : FloatingLabelBehavior.never,
      border: (widget.decoration != null)
          ? widget.decoration.border
          : OutlineInputBorder(
              borderSide: const BorderSide(
                color: const Color.fromRGBO(252, 0, 0, 1),
                style: BorderStyle.solid,
                width: 1,
              ),
              borderRadius: BorderRadius.circular(Const.RADIUS),
            ),
    );
  }

  @override
  Widget build(BuildContext context) {
    return GestureDetector(
      child: TextField(
        controller: editingController,
        focusNode: focusNode,
        decoration: decoration,
        keyboardType: widget.keyboardType,
        textInputAction: widget.textInputAction,
        textCapitalization: widget.textCapitalization,
        style: widget.style,
        strutStyle: widget.strutStyle,
        textAlign: widget.textAlign,
        textAlignVertical: widget.textAlignVertical,
        textDirection: widget.textDirection,
        readOnly: widget.readOnly,
        toolbarOptions: widget.toolbarOptions,
        showCursor: widget.showCursor,
        autofocus: widget.autofocus,
        obscuringCharacter: widget.obscuringCharacter,
        obscureText: widget.obscureText,
        autocorrect: widget.autocorrect,
        smartDashesType: widget.smartDashesType,
        smartQuotesType: widget.smartQuotesType,
        enableSuggestions: widget.enableSuggestions,
        maxLines: widget.maxLines,
        minLines: widget.minLines,
        expands: widget.expands,
        maxLength: widget.maxLength,
        maxLengthEnforced: widget.maxLengthEnforced,
        maxLengthEnforcement: widget.maxLengthEnforcement,
        onEditingComplete: () {
          if (widget.onEditingComplete != null) widget.onEditingComplete();
          if (widget.formControl != null) {
            FocusNode node = widget.formControl.getNextFocus(widget.name);
            if (node != null) {
              FocusScope.of(context).requestFocus(node);
            }
          }
        },
        onSubmitted: (String value) {
          if (widget.onSubmitted != null) widget.onSubmitted(value);
          if (widget.formControl != null) {
            FocusNode node = widget.formControl.getNextFocus(widget.name);
            if (node != null) {
              FocusScope.of(context).requestFocus(node);
            }
          }
        },
        onAppPrivateCommand: widget.onAppPrivateCommand,
        inputFormatters: widget.inputFormatters,
        cursorWidth: widget.cursorWidth,
        cursorHeight: widget.cursorHeight,
        cursorRadius: widget.cursorRadius,
        cursorColor: widget.cursorColor,
        keyboardAppearance: widget.keyboardAppearance,
        scrollPadding: widget.scrollPadding,
        dragStartBehavior: widget.dragStartBehavior,
        enableInteractiveSelection: widget.enableInteractiveSelection,
        selectionControls: widget.selectionControls,
        mouseCursor: widget.mouseCursor,
        buildCounter: widget.buildCounter,
        scrollController: widget.scrollController,
        scrollPhysics: widget.scrollPhysics,
        autofillHints: widget.autofillHints,
        restorationId: widget.restorationId,
        onTap: () {
          _selectDate(context);
          if (widget.onTap != null) {
            widget.onTap();
          }
        },
      ),
    );
  }

  String getValue(DateTime value) {
    if (value == null) return '';
    return "${value.toLocal()}".split(' ')[0];
  }
}
