import 'dart:io';
import 'dart:convert';

import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';
import 'package:flutter/services.dart';
import 'package:flutter_file_dialog/flutter_file_dialog.dart';
import './control.dart';

class FormFile extends StatefulWidget {
  final YekongaForm formControl;
  final String name;
  final String value;
  final bool isMultiple;
  final FocusNode focusNode;
  final InputDecoration decoration;
  final TextInputType keyboardType;
  final TextInputAction textInputAction;
  final TextCapitalization textCapitalization;
  final TextStyle style;
  final StrutStyle strutStyle;
  final TextAlign textAlign;
  final TextAlignVertical textAlignVertical;
  final TextDirection textDirection;
  final bool autofocus;
  final String obscuringCharacter;
  final bool obscureText;
  final bool autocorrect;
  final SmartDashesType smartDashesType;
  final SmartQuotesType smartQuotesType;
  final bool enableSuggestions;
  final bool isRequired;
  final int maxLines;
  final int minLines;
  final bool expands;
  final bool readOnly;
  final ToolbarOptions toolbarOptions;
  final bool showCursor;
  static const int noMaxLength = -1;
  final int maxLength;
  final bool maxLengthEnforced;
  final MaxLengthEnforcement maxLengthEnforcement;
  final ValueChanged<Map<String, dynamic>> onChanged;
  final VoidCallback onEditingComplete;
  final ValueChanged<String> onSubmitted;
  final AppPrivateCommandCallback onAppPrivateCommand;
  final List<TextInputFormatter> inputFormatters;
  final bool enabled;
  final double cursorWidth;
  final double cursorHeight;
  final Radius cursorRadius;
  final Color cursorColor;
  final Brightness keyboardAppearance;
  final EdgeInsets scrollPadding;
  final bool enableInteractiveSelection;
  final TextSelectionControls selectionControls;
  final DragStartBehavior dragStartBehavior;
  bool get selectionEnabled => enableInteractiveSelection;
  final GestureTapCallback onTap;
  final MouseCursor mouseCursor;
  final InputCounterWidgetBuilder buildCounter;
  final ScrollPhysics scrollPhysics;
  final ScrollController scrollController;
  final Iterable<String> autofillHints;
  final String restorationId;

  FormFile({
    Key key,
    this.formControl,
    this.value,
    @required this.name,
    this.isMultiple = false,
    this.focusNode,
    this.decoration = const InputDecoration(),
    this.keyboardType = TextInputType.text,
    this.textInputAction,
    this.textCapitalization = TextCapitalization.none,
    this.style,
    this.strutStyle,
    this.textAlign = TextAlign.start,
    this.textAlignVertical,
    this.textDirection,
    this.isRequired = false,
    this.readOnly = false,
    this.toolbarOptions,
    this.showCursor,
    this.autofocus = false,
    this.obscuringCharacter = '•',
    this.obscureText = false,
    this.autocorrect = true,
    this.smartDashesType = SmartDashesType.enabled,
    this.smartQuotesType = SmartQuotesType.enabled,
    this.enableSuggestions = true,
    this.maxLines = 1,
    this.minLines,
    this.expands = false,
    this.maxLength,
    this.maxLengthEnforced = true,
    this.maxLengthEnforcement,
    this.onChanged,
    this.onEditingComplete,
    this.onSubmitted,
    this.onAppPrivateCommand,
    this.inputFormatters,
    this.enabled = false,
    this.cursorWidth = 2.0,
    this.cursorHeight,
    this.cursorRadius,
    this.cursorColor,
    this.keyboardAppearance,
    this.scrollPadding = const EdgeInsets.all(20.0),
    this.dragStartBehavior = DragStartBehavior.start,
    this.enableInteractiveSelection = true,
    this.selectionControls,
    this.onTap,
    this.mouseCursor,
    this.buildCounter,
    this.scrollController,
    this.scrollPhysics,
    this.autofillHints,
    this.restorationId,
  }) : super(key: key);

  @override
  _FormFileState createState() => _FormFileState();
}

class _FormFileState extends State<FormFile> {
  Map<String, dynamic> innerValue;
  FocusNode focusNode;
  bool isValid = true;
  InputDecoration decoration;
  String displayValue = "";
  TextEditingController editingController = new TextEditingController();

  Future<void> _selectFile(BuildContext context) async {
    dynamic value;

    final params = OpenFileDialogParams(
      dialogType: OpenFileDialogType.image,
      sourceType: SourceType.photoLibrary,
    );
    final String filePath = await FlutterFileDialog.pickFile(params: params);

    // final FilePickerResult result = await FilePicker.platform.pickFiles(
    // 	allowMultiple: widget.isMultiple,
    // 	type: FileType.custom,
    // 	allowedExtensions: ['jpg', 'pdf', 'doc'],
    // );

    // if(result != null) {
    // 	if(widget.isMultiple) {
    // 		value = result.files.map((file) => getFileValue(file)).toList();
    // 	} else {
    // 		value = getFileValue(result.files.first);
    // 	}
    // } else {
    // // User canceled the picker
    // }

    if (widget.isMultiple) {
      // value = result.files.map((file) => getFileValue(file)).toList();
    } else {
      value = getFileValue(filePath);
    }

    if (value != null && value != innerValue) {
      setState(() {
        innerValue = value;
        widget.formControl.setValue(widget.name, value);
        widget.onChanged(value);
      });
    }
  }

  Map<String, dynamic> getFileValue(String filePath, {bool isMultiple: false}) {
    final file = File(filePath);
    final bytes = file.readAsBytesSync();

    String base64 = base64Encode(bytes);

    if (widget.isMultiple) {
      // displayValue += "${file.name}";
    } else {
      displayValue = filePath;
    }

    editingController.text = displayValue;

    return {
      "fileName": file.path,
      "fileSize": 0,
      "fileType": "file",
      "base64": base64,
    };
  }

  @override
  void initState() {
    super.initState();
    dynamic value = getValue(innerValue);
    if (widget.formControl != null) {
      if (widget.formControl.containsKey(widget.name)) {
        value = widget.formControl.getValue(widget.name);
      } else {
        widget.formControl.setValue(widget.name, value);
      }

      widget.formControl.setClearFunction(widget.name, () {
        innerValue = null;
        editingController.text = "";
        setState(() {});
      });

      focusNode = widget.focusNode ?? FocusNode();
      widget.formControl
          .setValidationRequirement(widget.name, widget.isRequired);
      widget.formControl.setFocusNode(widget.name, focusNode);
    }

    innerValue = value;
    editingController.text = value['fileName'];

    if (widget.decoration != null) {
      decoration = InputDecoration(
          suffixIcon: Padding(
        padding: const EdgeInsetsDirectional.only(end: 12.0),
        child: Icon(
            Icons.calendar_today_outlined), // myIcon is a 48px-wide widget.
      ));
    } else {
      decoration = widget.decoration;
    }
  }

  @override
  Widget build(BuildContext context) {
    return TextField(
      controller: editingController,
      focusNode: focusNode,
      decoration: decoration,
      keyboardType: widget.keyboardType,
      textInputAction: widget.textInputAction,
      textCapitalization: widget.textCapitalization,
      style: widget.style,
      strutStyle: widget.strutStyle,
      textAlign: widget.textAlign,
      textAlignVertical: widget.textAlignVertical,
      textDirection: widget.textDirection,
      readOnly: widget.readOnly,
      toolbarOptions: widget.toolbarOptions,
      showCursor: widget.showCursor,
      autofocus: widget.autofocus,
      obscuringCharacter: widget.obscuringCharacter,
      obscureText: widget.obscureText,
      autocorrect: widget.autocorrect,
      smartDashesType: widget.smartDashesType,
      smartQuotesType: widget.smartQuotesType,
      enableSuggestions: widget.enableSuggestions,
      maxLines: widget.maxLines,
      minLines: widget.minLines,
      expands: widget.expands,
      maxLength: widget.maxLength,
      maxLengthEnforced: widget.maxLengthEnforced,
      maxLengthEnforcement: widget.maxLengthEnforcement,
      onEditingComplete: widget.onEditingComplete,
      onSubmitted: widget.onSubmitted,
      onAppPrivateCommand: widget.onAppPrivateCommand,
      inputFormatters: widget.inputFormatters,
      enabled: widget.enabled,
      cursorWidth: widget.cursorWidth,
      cursorHeight: widget.cursorHeight,
      cursorRadius: widget.cursorRadius,
      cursorColor: widget.cursorColor,
      keyboardAppearance: widget.keyboardAppearance,
      scrollPadding: widget.scrollPadding,
      dragStartBehavior: widget.dragStartBehavior,
      enableInteractiveSelection: widget.enableInteractiveSelection,
      selectionControls: widget.selectionControls,
      onTap: () {
        _selectFile(context);
        if (widget.onTap != null) {
          widget.onTap();
        }
      },
      mouseCursor: widget.mouseCursor,
      buildCounter: widget.buildCounter,
      scrollController: widget.scrollController,
      scrollPhysics: widget.scrollPhysics,
      autofillHints: widget.autofillHints,
      restorationId: widget.restorationId,
    );
  }

  Map<String, dynamic> getValue(dynamic value) {
    Map<String, dynamic> result;
    if (value is String) {
      result = {
        "fileName": value,
        "fileSize": 0,
        "fileType": "url",
        "base64": value,
      };
    } else if (value is Map<String, dynamic>) {
      result = value;
    } else {
      result = {
        "fileName": null,
        "fileSize": null,
        "fileType": null,
        "base64": null,
      };
    }

    return result;
  }
}
