import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';
import './control.dart';

class FormSlider extends StatefulWidget {
  final String name;
  final double value;
  final YekongaForm formControl;
  final ValueChanged<double> onChanged;
  final ValueChanged<double> onChangeStart;
  final ValueChanged<double> onChangeEnd;
  final double min;
  final double max;
  final int divisions;
  final bool isRequired;
  final String label;
  final Color activeColor;
  final Color inactiveColor;
  final MouseCursor mouseCursor;
  final SemanticFormatterCallback semanticFormatterCallback;
  final FocusNode focusNode;
  bool isValid = true;
  InputDecoration decoration;
  final bool autofocus;

  FormSlider({
    Key key,
    this.formControl,
    @required this.name,
    @required this.value,
    @required this.onChanged,
    this.onChangeStart,
    this.onChangeEnd,
    this.min = 0.0,
    this.max = 1.0,
    this.divisions,
    this.label,
    this.activeColor,
    this.inactiveColor,
    this.mouseCursor,
    this.semanticFormatterCallback,
    this.focusNode,
    this.autofocus = false,
    this.isRequired = false,
  }) : super(key: key);

  @override
  _FormSliderState createState() => _FormSliderState();
}

class _FormSliderState extends State<FormSlider> {
  double innerValue = 0;
  FocusNode focusNode;
  bool isValid = true;
  InputDecoration decoration;
  TextEditingController editingController = new TextEditingController();

  @override
  void initState() {
    super.initState();
    if (widget.formControl.containsKey(widget.name)) {
      innerValue = widget.formControl.getValue(widget.name);
    } else {
      innerValue = widget.value;
    }
  }

  @override
  Widget build(BuildContext context) {
    return Slider(
      value: innerValue,
      onChanged: (value) {
        setState(() {
          innerValue = value;
          widget.formControl.setValue(widget.name, innerValue);
          widget.onChanged(innerValue);
        });
      },
      onChangeStart: widget.onChangeStart,
      onChangeEnd: widget.onChangeEnd,
      min: widget.min,
      max: widget.max,
      divisions: widget.divisions,
      label: widget.label,
      activeColor: widget.activeColor,
      inactiveColor: widget.inactiveColor,
      mouseCursor: widget.mouseCursor,
      semanticFormatterCallback: widget.semanticFormatterCallback,
      focusNode: focusNode,
      autofocus: widget.autofocus,
    );
  }
}
