import 'package:flutter/material.dart';
import './control.dart';

class FormRadio extends StatefulWidget {
  final String name;
  final String value;
  final String groupValue;
  final ValueChanged<String> onChanged;
  final Color activeColor;
  final Color tileColor;
  final Widget title;
  final Widget subtitle;
  final Widget secondary;
  final bool isThreeLine;
  final bool isRequired;
  final bool dense;
  final bool selected;
  final ListTileControlAffinity controlAffinity;
  final bool autofocus;
  final EdgeInsetsGeometry contentPadding;
  final ShapeBorder shape;
  final Color selectedTileColor;
  final YekongaForm formControl;

  FormRadio({
    Key key,
    this.formControl,
    @required this.name,
    @required this.value,
    @required this.groupValue,
    @required this.onChanged,
    this.activeColor,
    this.tileColor,
    this.title,
    this.subtitle,
    this.isThreeLine = false,
    this.dense,
    this.secondary,
    this.selected = false,
    this.controlAffinity = ListTileControlAffinity.platform,
    this.autofocus = false,
    this.isRequired = false,
    this.contentPadding = EdgeInsets.zero,
    this.shape,
    this.selectedTileColor,
  }) : super(key: key);

  @override
  _FormRadioState createState() => _FormRadioState();
}

class _FormRadioState extends State<FormRadio> {
  String innerValue;
  FocusNode focusNode;
  bool isValid = true;
  InputDecoration decoration;
  TextEditingController editingController = new TextEditingController();

  @override
  void initState() {
    super.initState();
    if (widget.formControl.containsKey(widget.name)) {
      innerValue = widget.formControl.getValue(widget.name);
    } else {
      innerValue = widget.value;
    }
  }

  @override
  Widget build(BuildContext context) {
    return RadioListTile(
      value: innerValue,
      groupValue: widget.groupValue,
      onChanged: (value) {
        setState(() {
          innerValue = value.toString();
          widget.formControl.setValue(widget.name, innerValue);
          widget.onChanged(innerValue);
        });
      },
      activeColor: widget.activeColor,
      tileColor: widget.tileColor,
      title: widget.title,
      subtitle: widget.subtitle,
      isThreeLine: widget.isThreeLine,
      dense: widget.dense,
      secondary: widget.secondary,
      selected: widget.selected,
      controlAffinity: widget.controlAffinity,
      autofocus: widget.autofocus,
      contentPadding: widget.contentPadding,
      shape: widget.shape,
      selectedTileColor: widget.selectedTileColor,
    );
  }
}
