import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';
import 'package:flutter_rating_bar/flutter_rating_bar.dart';
import './control.dart';

class FormRatingBar extends StatefulWidget {
  final String name;
  final double value;
  final YekongaForm formControl;
  final ValueChanged<double> onChanged;
  final IndexedWidgetBuilder itemBuilder;

  final Color glowColor;
  final double maxRating;
  final TextDirection textDirection;
  final Color unratedColor;
  final bool allowHalfRating;
  final Axis direction;
  final bool glow;
  final double glowRadius;
  final bool ignoreGestures;
  final double initialRating;
  final int itemCount;
  final EdgeInsetsGeometry itemPadding;
  final double itemSize;
  final double minRating;
  final bool tapOnlyMode;
  final bool isRequired;
  final bool updateOnDrag;
  final WrapAlignment wrapAlignment;

  FormRatingBar({
    Key key,
    this.formControl,
    @required this.name,
    @required this.value,
    @required this.onChanged,
    this.itemBuilder,
    this.glowColor,
    this.maxRating,
    this.textDirection,
    this.unratedColor,
    this.allowHalfRating = false,
    this.direction = Axis.horizontal,
    this.glow = true,
    this.glowRadius = 2,
    this.ignoreGestures = false,
    this.initialRating = 0.0,
    this.itemCount = 5,
    this.itemPadding = EdgeInsets.zero,
    this.itemSize = 40.0,
    this.minRating = 0,
    this.tapOnlyMode = false,
    this.updateOnDrag = false,
    this.isRequired = false,
    this.wrapAlignment = WrapAlignment.start,
  }) : super(key: key);

  @override
  _FormRatingBarState createState() => _FormRatingBarState();
}

class _FormRatingBarState extends State<FormRatingBar> {
  double innerValue = 0;
  FocusNode focusNode;
  bool isValid = true;
  InputDecoration decoration;
  TextEditingController editingController = new TextEditingController();

  @override
  void initState() {
    super.initState();
    if (widget.formControl.containsKey(widget.name)) {
      innerValue = widget.formControl.getValue(widget.name);
    } else {
      innerValue = widget.value;
    }
  }

  @override
  Widget build(BuildContext context) {
    IndexedWidgetBuilder _itemBuilder;
    if (widget.itemBuilder == null) {
      _itemBuilder = (context, i) {
        return Icon(
          Icons.star,
          color: Colors.amber,
        );
      };
    } else {
      _itemBuilder = widget.itemBuilder;
    }

    return RatingBar.builder(
      itemBuilder: _itemBuilder,
      onRatingUpdate: (value) {
        setState(() {
          innerValue = value;
          widget.formControl.setValue(widget.name, innerValue);
          widget.onChanged(innerValue);
        });
      },
      glowColor: widget.glowColor,
      maxRating: widget.maxRating,
      textDirection: widget.textDirection,
      unratedColor: widget.unratedColor,
      allowHalfRating: widget.allowHalfRating,
      direction: widget.direction,
      glow: widget.glow,
      glowRadius: widget.glowRadius,
      ignoreGestures: widget.ignoreGestures,
      initialRating: widget.initialRating,
      itemCount: widget.itemCount,
      itemPadding: widget.itemPadding,
      itemSize: widget.itemSize,
      minRating: widget.minRating,
      tapOnlyMode: widget.tapOnlyMode,
      updateOnDrag: widget.updateOnDrag,
      wrapAlignment: widget.wrapAlignment,
    );
  }
}
