import 'package:uuid/uuid.dart';
import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';
import 'package:flutter/services.dart';
import './control.dart';

class FormSelect extends StatefulWidget {
  final YekongaForm formControl;
  final String name;
  final String value;
  final FocusNode focusNode;
  final List<Map<String, dynamic>> items;
  final Function<Widget>(Map<String, dynamic>) itemTemplate;
  final InputDecoration decoration;
  final TextInputType keyboardType;
  final TextInputAction textInputAction;
  final TextCapitalization textCapitalization;
  final TextStyle style;
  final StrutStyle strutStyle;
  final TextAlign textAlign;
  final TextAlignVertical textAlignVertical;
  final TextDirection textDirection;
  final bool autofocus;
  final String obscuringCharacter;
  final bool obscureText;
  final bool autocorrect;
  final SmartDashesType smartDashesType;
  final SmartQuotesType smartQuotesType;
  final bool enableSuggestions;
  final int maxLines;
  final int minLines;
  final bool expands;
  final bool readOnly;
  final ToolbarOptions toolbarOptions;
  final bool showCursor;
  static const int noMaxLength = -1;
  final int maxLength;
  final bool maxLengthEnforced;
  final MaxLengthEnforcement maxLengthEnforcement;
  final ValueChanged<String> onChanged;
  final VoidCallback onEditingComplete;
  final ValueChanged<String> onSubmitted;
  final AppPrivateCommandCallback onAppPrivateCommand;
  final List<TextInputFormatter> inputFormatters;
  final bool isRequired;
  final bool enabled;
  final double cursorWidth;
  final double cursorHeight;
  final Radius cursorRadius;
  final Color cursorColor;
  final Brightness keyboardAppearance;
  final EdgeInsets scrollPadding;
  final bool enableInteractiveSelection;
  final TextSelectionControls selectionControls;
  final DragStartBehavior dragStartBehavior;
  bool get selectionEnabled => enableInteractiveSelection;
  final GestureTapCallback onTap;
  final MouseCursor mouseCursor;
  final InputCounterWidgetBuilder buildCounter;
  final ScrollPhysics scrollPhysics;
  final ScrollController scrollController;
  final Iterable<String> autofillHints;
  final String restorationId;

  FormSelect({
    Key key,
    this.formControl,
    this.value,
    @required this.name,
    this.focusNode,
    this.items,
    this.itemTemplate,
    this.decoration = const InputDecoration(),
    this.keyboardType = TextInputType.text,
    this.textInputAction,
    this.textCapitalization = TextCapitalization.none,
    this.style,
    this.strutStyle,
    this.textAlign = TextAlign.start,
    this.textAlignVertical,
    this.textDirection,
    this.readOnly = false,
    this.toolbarOptions,
    this.showCursor,
    this.autofocus = false,
    this.isRequired = false,
    this.obscuringCharacter = '•',
    this.obscureText = false,
    this.autocorrect = true,
    this.smartDashesType = SmartDashesType.enabled,
    this.smartQuotesType = SmartQuotesType.enabled,
    this.enableSuggestions = true,
    this.maxLines = 1,
    this.minLines,
    this.expands = false,
    this.maxLength,
    this.maxLengthEnforced = true,
    this.maxLengthEnforcement,
    this.onChanged,
    this.onEditingComplete,
    this.onSubmitted,
    this.onAppPrivateCommand,
    this.inputFormatters,
    this.enabled = false,
    this.cursorWidth = 2.0,
    this.cursorHeight,
    this.cursorRadius,
    this.cursorColor,
    this.keyboardAppearance,
    this.scrollPadding = const EdgeInsets.all(20.0),
    this.dragStartBehavior = DragStartBehavior.start,
    this.enableInteractiveSelection = true,
    this.selectionControls,
    this.onTap,
    this.mouseCursor,
    this.buildCounter,
    this.scrollController,
    this.scrollPhysics,
    this.autofillHints,
    this.restorationId,
  }) : super(key: key);

  @override
  _FormSelectState createState() => _FormSelectState();
}

class _FormSelectState extends State<FormSelect> {
  dynamic innerValue;
  String displayValue;
  FocusNode focusNode;
  bool isValid = true;
  InputDecoration decoration;
  TextEditingController editingController = new TextEditingController();
  String groupValue = Uuid().v1();

  AlertDialog itemsDialog(BuildContext context) {
    final AlertDialog dialog = AlertDialog(
      title: Text('Select'),
      contentPadding: EdgeInsets.zero,
      content: Column(
          mainAxisSize: MainAxisSize.min,
          children: List<Widget>.generate(widget.items.length, (i) {
            final Map<String, dynamic> item = widget.items[i];
            final Color selected = (item["value"].toString() == innerValue)
                ? Theme.of(context).primaryColor.withOpacity(0.3)
                : Colors.transparent;

            return GestureDetector(
              child: Container(
                color: selected,
                child: (widget.itemTemplate != null)
                    ? widget.itemTemplate(item)
                    : Row(
                        crossAxisAlignment: CrossAxisAlignment.center,
                        children: [
                            Radio(
                              value: item["value"].toString(),
                              groupValue: groupValue,
                              onChanged: (value) {
                                setState(() {
                                  innerValue = value;
                                  widget.formControl
                                      .setValue(widget.name, innerValue);
                                  widget.onChanged(innerValue);
                                  editingController.text = item["label"];
                                });
                              },
                            ),
                            Text(item["label"]),
                          ]),
              ),
              onTap: () {
                if (widget.itemTemplate != null) {
                  setState(() {
                    innerValue = item["value"].toString();
                    widget.formControl.setValue(widget.name, innerValue);
                    widget.onChanged(innerValue);
                    editingController.text = item["label"];
                  });
                }
              },
            );
          }).toList()),
      actions: [
        ElevatedButton(
          onPressed: () => Navigator.pop(context),
          child: Text('CANCEL'),
        ),
      ],
    );

    return dialog;
  }

  Future<void> _selectItem(BuildContext context) async {
    String value;

    showDialog(
        context: context,
        builder: (_) {
          return itemsDialog(context);
        });
  }

  @override
  void initState() {
    super.initState();
    dynamic value;
    if (widget.formControl.containsKey(widget.name)) {
      value = widget.formControl.getValue(widget.name);
    } else {
      value = innerValue;
    }

    editingController.text = value.toString();

    if (widget.decoration != null) {
      decoration = InputDecoration(
          suffixIcon: Padding(
        padding: const EdgeInsetsDirectional.only(end: 12.0),
        child: Icon(
            Icons.calendar_today_outlined), // myIcon is a 48px-wide widget.
      ));
    } else {
      decoration = widget.decoration;
    }
  }

  @override
  Widget build(BuildContext context) {
    return TextField(
      controller: editingController,
      focusNode: focusNode,
      decoration: decoration,
      keyboardType: widget.keyboardType,
      textInputAction: widget.textInputAction,
      textCapitalization: widget.textCapitalization,
      style: widget.style,
      strutStyle: widget.strutStyle,
      textAlign: widget.textAlign,
      textAlignVertical: widget.textAlignVertical,
      textDirection: widget.textDirection,
      readOnly: widget.readOnly,
      toolbarOptions: widget.toolbarOptions,
      showCursor: widget.showCursor,
      autofocus: widget.autofocus,
      obscuringCharacter: widget.obscuringCharacter,
      obscureText: widget.obscureText,
      autocorrect: widget.autocorrect,
      smartDashesType: widget.smartDashesType,
      smartQuotesType: widget.smartQuotesType,
      enableSuggestions: widget.enableSuggestions,
      maxLines: widget.maxLines,
      minLines: widget.minLines,
      expands: widget.expands,
      maxLength: widget.maxLength,
      maxLengthEnforced: widget.maxLengthEnforced,
      maxLengthEnforcement: widget.maxLengthEnforcement,
      onEditingComplete: widget.onEditingComplete,
      onSubmitted: widget.onSubmitted,
      onAppPrivateCommand: widget.onAppPrivateCommand,
      inputFormatters: widget.inputFormatters,
      enabled: widget.enabled,
      cursorWidth: widget.cursorWidth,
      cursorHeight: widget.cursorHeight,
      cursorRadius: widget.cursorRadius,
      cursorColor: widget.cursorColor,
      keyboardAppearance: widget.keyboardAppearance,
      scrollPadding: widget.scrollPadding,
      dragStartBehavior: widget.dragStartBehavior,
      enableInteractiveSelection: widget.enableInteractiveSelection,
      selectionControls: widget.selectionControls,
      onTap: () {
        _selectItem(context);
        if (widget.onTap != null) {
          widget.onTap();
        }
      },
      mouseCursor: widget.mouseCursor,
      buildCounter: widget.buildCounter,
      scrollController: widget.scrollController,
      scrollPhysics: widget.scrollPhysics,
      autofillHints: widget.autofillHints,
      restorationId: widget.restorationId,
    );
  }

  String getValue(DateTime value) {
    return "${value.toLocal()}".split(' ')[0];
  }
}
