import "package:intl/intl.dart" as Intl;
import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';
import 'package:flutter/services.dart';
import './control.dart';

class FormTimePicker extends StatefulWidget {
  final YekongaForm formControl;
  final String name;
  final String value;
  final FocusNode focusNode;
  final InputDecoration decoration;
  final TextInputType keyboardType;
  final TextInputAction textInputAction;
  final TextCapitalization textCapitalization;
  final TextStyle style;
  final StrutStyle strutStyle;
  final TextAlign textAlign;
  final TextAlignVertical textAlignVertical;
  final TextDirection textDirection;
  final bool isRequired;
  final bool autofocus;
  final String obscuringCharacter;
  final bool obscureText;
  final bool autocorrect;
  final SmartDashesType smartDashesType;
  final SmartQuotesType smartQuotesType;
  final bool enableSuggestions;
  final int maxLines;
  final int minLines;
  final bool expands;
  final bool readOnly;
  final ToolbarOptions toolbarOptions;
  final bool showCursor;
  static const int noMaxLength = -1;
  final int maxLength;
  final bool maxLengthEnforced;
  final MaxLengthEnforcement maxLengthEnforcement;
  final ValueChanged<String> onChanged;
  final VoidCallback onEditingComplete;
  final ValueChanged<String> onSubmitted;
  final AppPrivateCommandCallback onAppPrivateCommand;
  final List<TextInputFormatter> inputFormatters;
  final bool enabled;
  final double cursorWidth;
  final double cursorHeight;
  final Radius cursorRadius;
  final Color cursorColor;
  final Brightness keyboardAppearance;
  final EdgeInsets scrollPadding;
  final bool enableInteractiveSelection;
  final TextSelectionControls selectionControls;
  final DragStartBehavior dragStartBehavior;
  bool get selectionEnabled => enableInteractiveSelection;
  final GestureTapCallback onTap;
  final MouseCursor mouseCursor;
  final InputCounterWidgetBuilder buildCounter;
  final ScrollPhysics scrollPhysics;
  final ScrollController scrollController;
  final Iterable<String> autofillHints;
  final String restorationId;

  FormTimePicker({
    Key key,
    this.formControl,
    this.value,
    @required this.name,
    this.focusNode,
    this.decoration = const InputDecoration(),
    this.keyboardType = TextInputType.text,
    this.textInputAction,
    this.textCapitalization = TextCapitalization.none,
    this.style,
    this.strutStyle,
    this.textAlign = TextAlign.start,
    this.textAlignVertical,
    this.textDirection,
    this.readOnly = false,
    this.isRequired = false,
    this.toolbarOptions,
    this.showCursor,
    this.autofocus = false,
    this.obscuringCharacter = '•',
    this.obscureText = false,
    this.autocorrect = true,
    this.smartDashesType = SmartDashesType.enabled,
    this.smartQuotesType = SmartQuotesType.enabled,
    this.enableSuggestions = true,
    this.maxLines = 1,
    this.minLines,
    this.expands = false,
    this.maxLength,
    this.maxLengthEnforced = true,
    this.maxLengthEnforcement,
    this.onChanged,
    this.onEditingComplete,
    this.onSubmitted,
    this.onAppPrivateCommand,
    this.inputFormatters,
    this.enabled = false,
    this.cursorWidth = 2.0,
    this.cursorHeight,
    this.cursorRadius,
    this.cursorColor,
    this.keyboardAppearance,
    this.scrollPadding = const EdgeInsets.all(20.0),
    this.dragStartBehavior = DragStartBehavior.start,
    this.enableInteractiveSelection = true,
    this.selectionControls,
    this.onTap,
    this.mouseCursor,
    this.buildCounter,
    this.scrollController,
    this.scrollPhysics,
    this.autofillHints,
    this.restorationId,
  }) : super(key: key);

  @override
  _FormTimePickerState createState() => _FormTimePickerState();
}

class _FormTimePickerState extends State<FormTimePicker> {
  TimeOfDay innerValue = TimeOfDay.now();
  InputDecoration decoration;
  FocusNode focusNode;
  bool isValid = true;
  TextEditingController editingController = new TextEditingController();

  Future<void> _selectDate(BuildContext context) async {
    final TimeOfDay value = await showTimePicker(
      context: context,
      initialTime: innerValue,
    );

    if (value != null && value != innerValue) {
      setState(() {
        innerValue = value;
        String strValue = getValue(innerValue);
        widget.formControl.setValue(widget.name, strValue);
        widget.onChanged(strValue);
        editingController.text = strValue;
      });
    }
  }

  @override
  void initState() {
    super.initState();
    dynamic value;

    if (widget.formControl != null) {
      if (widget.formControl.containsKey(widget.name)) {
        value = widget.formControl.getValue(widget.name);
      } else {
        widget.formControl.setValue(widget.name, widget.value);
        value = widget.value;
      }

      widget.formControl.setClearFunction(widget.name, () {
        innerValue = null;
        editingController.text = "";
        setState(() {});
      });

      focusNode = widget.focusNode ?? FocusNode();
      widget.formControl
          .setValidationRequirement(widget.name, widget.isRequired);
      widget.formControl.setFocusNode(widget.name, focusNode);
    }

    editingController.text = value.toString();

    if (value is String) {
      innerValue = setValue(value);
    }

    if (widget.decoration != null) {
      decoration = InputDecoration(
          suffixIcon: Padding(
        padding: const EdgeInsetsDirectional.only(end: 12.0),
        child: Icon(
            Icons.calendar_today_outlined), // myIcon is a 48px-wide widget.
      ));
    } else {
      decoration = widget.decoration;
    }
  }

  @override
  Widget build(BuildContext context) {
    return TextField(
      controller: editingController,
      focusNode: focusNode,
      decoration: decoration,
      keyboardType: widget.keyboardType,
      textInputAction: widget.textInputAction,
      textCapitalization: widget.textCapitalization,
      style: widget.style,
      strutStyle: widget.strutStyle,
      textAlign: widget.textAlign,
      textAlignVertical: widget.textAlignVertical,
      textDirection: widget.textDirection,
      readOnly: widget.readOnly,
      toolbarOptions: widget.toolbarOptions,
      showCursor: widget.showCursor,
      autofocus: widget.autofocus,
      obscuringCharacter: widget.obscuringCharacter,
      obscureText: widget.obscureText,
      autocorrect: widget.autocorrect,
      smartDashesType: widget.smartDashesType,
      smartQuotesType: widget.smartQuotesType,
      enableSuggestions: widget.enableSuggestions,
      maxLines: widget.maxLines,
      minLines: widget.minLines,
      expands: widget.expands,
      maxLength: widget.maxLength,
      maxLengthEnforced: widget.maxLengthEnforced,
      maxLengthEnforcement: widget.maxLengthEnforcement,
      onEditingComplete: () {
        if (widget.onEditingComplete != null) widget.onEditingComplete();
        if (widget.formControl != null) {
          FocusNode node = widget.formControl.getNextFocus(widget.name);
          if (node != null) {
            FocusScope.of(context).requestFocus(node);
          }
        }
      },
      onSubmitted: (String value) {
        if (widget.onSubmitted != null) widget.onSubmitted(value);
        if (widget.formControl != null) {
          FocusNode node = widget.formControl.getNextFocus(widget.name);
          if (node != null) {
            FocusScope.of(context).requestFocus(node);
          }
        }
      },
      onAppPrivateCommand: widget.onAppPrivateCommand,
      inputFormatters: widget.inputFormatters,
      enabled: widget.enabled,
      cursorWidth: widget.cursorWidth,
      cursorHeight: widget.cursorHeight,
      cursorRadius: widget.cursorRadius,
      cursorColor: widget.cursorColor,
      keyboardAppearance: widget.keyboardAppearance,
      scrollPadding: widget.scrollPadding,
      dragStartBehavior: widget.dragStartBehavior,
      enableInteractiveSelection: widget.enableInteractiveSelection,
      selectionControls: widget.selectionControls,
      onTap: () {
        _selectDate(context);
        if (widget.onTap != null) {
          widget.onTap();
        }
      },
      mouseCursor: widget.mouseCursor,
      buildCounter: widget.buildCounter,
      scrollController: widget.scrollController,
      scrollPhysics: widget.scrollPhysics,
      autofillHints: widget.autofillHints,
      restorationId: widget.restorationId,
    );
  }

  String getValue(TimeOfDay value) {
    final f = new Intl.NumberFormat("00");

    return "${f.format(value.hour)}:${f.format(value.minute)}";
  }

  TimeOfDay setValue(String value) {
    List<String> time = value.split(':');

    if (time.length == 2) {
      return TimeOfDay(
          hour: int.parse(time.first), minute: int.parse(time.last));
    }

    return TimeOfDay.now();
  }
}
